@@ -2,8 +2,8 @@ import numpy as np
 import pandas as pd
 
 
-def precision(predictions, label_ids, id2tkn, id2type, only_age=False, prediction_scope=True, ignore_ids={-100}, shifted_labels=False, predictions_are_scores=True,
-              old_data=None, topk=1):
+def precision(predictions, label_ids, id2tkn, id2type, only_age=False, prediction_scope=True, ignore_ids={-100}, shifted_labels=False, 
+        predictions_are_scores=True, old_data=None, topk=1, start=0):
     r''' Calculate precision for next concept prediction.
 
     Args:
@@ -11,14 +11,18 @@ def precision(predictions, label_ids, id2tkn, id2type, only_age=False, predictio
             Expected shape <batch_size> x <sequence_length> x <vocabulary_size>
         label_ids:
             Expected shape <batch_size> x <sequence_length>
+        id2tkn:
+            Map from ID to tokens
+        id2type:
+            Map from ID to token type (e.g. `age`, `cui`, ...)
         only_age:
             Calcualte only for tokens that represent age
         prediction_scope:
             How much into the future should we look to accept something as correct:
                 - `one` has to be the next concept
                 - `age` until the next age token
-                - `any` whnever
-        ignore_id:
+                - `any` whenever
+        ignore_ids:
             label_ids with this IDs will be ignored
         shifted_labels:
             Are labels == input_ids, or shifted by one to the left
@@ -28,6 +32,8 @@ def precision(predictions, label_ids, id2tkn, id2type, only_age=False, predictio
             If set it will load old values for tp/fp/positives/negatives and continue ontop of those
         topk:
             How many predicted labels to consider when calculating precision
+        start:
+            At what point to start - we will look only at the precision of concepts at positions after start
 
     Return (Dict[str, ]):
         precision:
@@ -45,13 +51,15 @@ def precision(predictions, label_ids, id2tkn, id2type, only_age=False, predictio
         outputs = np.argsort(-1 * predictions, axis=2)
     else:
         outputs = predictions
+
     tp = 0
     fp = 0
     positives = {}
     negatives = {}
-    start = 1
-    if shifted_labels:
-        start = 0
+    # If not shifted_labels label = prediction - 1
+    label_position_shift = 0 if shifted_labels else 1
+    # If labels are not shifted move the start by one
+    start += 0 if shifted_labels else 1
 
     if old_data:
         tp = old_data['tp']
@@ -76,23 +84,22 @@ def precision(predictions, label_ids, id2tkn, id2type, only_age=False, predictio
             return end
 
     for ind, lbl in enumerate(label_ids):
-        for i in range(start, len(lbl)):
-            # We start from 1 because there is no prediction of token at position 0, that
-            #is the start token
-            tkn = str(id2tkn.get(lbl[i], lbl[i]))
-            is_age = True if tkn.isdecimal() and int(tkn) >= 0 and int(tkn) <= 300 else False
-            if lbl[i] not in ignore_ids:
-                if (is_age and only_age) or (not is_age and not only_age):
-                    end = prediction_end_index(i, lbl)
+        if start < len(lbl):
+            for i in range(start, len(lbl)):
+                tkn = str(id2tkn.get(lbl[i], lbl[i]))
+                is_age = True if tkn.isdecimal() and int(tkn) >= 0 and int(tkn) <= 300 else False
+                if lbl[i] not in ignore_ids:
+                    if (is_age and only_age) or (not is_age and not only_age):
+                        end = prediction_end_index(i, lbl)
 
-                    # If predictions are scores we can do topk, if not just do simple label match
-                    if (predictions_are_scores and any([out in lbl[i:end] for out in outputs[ind][i-start][0:topk]])) or \
-                        (not predictions_are_scores and outputs[ind][i-start] in lbl[i:end]):
-                        tp += 1
-                        positives[tkn] = positives.get(tkn, 0) + 1
-                    else:
-                        fp += 1
-                        negatives[tkn] = negatives.get(tkn, 0) + 1
+                        # If predictions are scores we can do topk, if not just do simple label match
+                        if (predictions_are_scores and any([out in lbl[i:end] for out in outputs[ind][i-label_position_shift][0:topk]])) or \
+                            (not predictions_are_scores and outputs[ind][i-label_position_shift] in lbl[i:end]):
+                            tp += 1
+                            positives[tkn] = positives.get(tkn, 0) + 1
+                        else:
+                            fp += 1
+                            negatives[tkn] = negatives.get(tkn, 0) + 1
 
     prec = tp / (tp + fp)
 
@@ -117,3 +124,30 @@ def sort_precision_output(precision_output, cdb, main='positives'):
     out = pd.DataFrame(out, columns=['precision', 'name', main, other, 'cui'])
 
     return out
+
+
+class ComputePrecisionHF(object):
+    r''' Used for computing precison when working with HF trainer
+    '''
+
+    def __init__(self, id2tkn, id2type, batch_size=1000, **kwargs):
+        self.id2tkn = id2tkn
+        self.id2type = id2type
+        self.batch_size = batch_size
+        self.kwargs = kwargs
+
+    def __call__(self, p):
+        # We will do this in batches, because it can be very memory demanding
+        metrics_data = None
+        start = 0
+        print(self.kwargs)
+        while start < len(p.predictions):
+            predictions = np.argmax(p.predictions[start:start+self.batch_size], axis=2) # For speed do not send predictions as scores
+            label_ids = p.label_ids[start:start+self.batch_size]
+            metrics_data = precision(predictions, label_ids=label_ids, id2tkn=self.id2tkn, id2type=self.id2type, old_data=metrics_data,
+                                     predictions_are_scores=False, **self.kwargs)
+            start += self.batch_size
+
+        return {
+            'precision': metrics_data['precision'],
+        }
